@@ -1,27 +1,35 @@
 import os
 
-from .compat import json
+from .compat import json as simplejson
 from .endpoint import ApiEndpoint
 from .objects import (
   ApiObject,
   BestAssignments,
+  Checkpoint,
   Client,
   Experiment,
+  Importances,
+  MetricImportances,
   Observation,
+  Organization,
   Pagination,
   Plan,
-  Suggestion,
+  Project,
+  QueuedSuggestion,
+  Session,
   StoppingCriteria,
+  Suggestion,
   Token,
+  TrainingRun,
 )
 from .requestor import Requestor, DEFAULT_API_URL
 from .resource import ApiResource
 from .version import VERSION
 
 class ConnectionImpl(object):
-  def __init__(self, requestor):
+  def __init__(self, requestor, api_url=None):
     self.requestor = requestor
-    self.api_url = DEFAULT_API_URL
+    self.api_url = api_url or DEFAULT_API_URL
 
     suggestions = ApiResource(
       self,
@@ -31,6 +39,16 @@ class ConnectionImpl(object):
         ApiEndpoint(None, object_or_paginated_objects(Suggestion), 'GET', 'fetch'),
         ApiEndpoint(None, Suggestion, 'PUT', 'update'),
         ApiEndpoint(None, None, 'DELETE', 'delete'),
+      ],
+    )
+
+    queued_suggestions = ApiResource(
+      self,
+      'queued_suggestions',
+      endpoints=[
+        ApiEndpoint(None, QueuedSuggestion, 'POST', 'create'),
+        ApiEndpoint(None, object_or_paginated_objects(QueuedSuggestion), 'GET', 'fetch'),
+        ApiEndpoint(None, None, 'DELETE', 'delete'),
       ]
     )
 
@@ -38,11 +56,12 @@ class ConnectionImpl(object):
       self,
       'observations',
       endpoints=[
+        ApiEndpoint('batch', paginated_objects(Observation), 'POST', 'create_batch'),
         ApiEndpoint(None, Observation, 'POST', 'create'),
         ApiEndpoint(None, object_or_paginated_objects(Observation), 'GET', 'fetch'),
         ApiEndpoint(None, Observation, 'PUT', 'update'),
         ApiEndpoint(None, None, 'DELETE', 'delete'),
-      ]
+      ],
     )
 
     plan = ApiResource(
@@ -58,6 +77,7 @@ class ConnectionImpl(object):
       'tokens',
       endpoints=[
         ApiEndpoint(None, Token, 'POST', 'create'),
+        ApiEndpoint(None, object_or_paginated_objects(Token), 'GET', 'fetch'),
       ],
     )
 
@@ -69,6 +89,22 @@ class ConnectionImpl(object):
       ],
     )
 
+    importances = ApiResource(
+      self,
+      'importances',
+      endpoints=[
+        ApiEndpoint(None, Importances, 'GET', 'fetch'),
+      ],
+    )
+
+    metric_importances = ApiResource(
+      self,
+      'metric_importances',
+      endpoints=[
+        ApiEndpoint(None, paginated_objects(MetricImportances), 'GET', 'fetch'),
+      ],
+    )
+
     stopping_criteria = ApiResource(
       self,
       'stopping_criteria',
@@ -77,6 +113,25 @@ class ConnectionImpl(object):
       ],
     )
 
+    checkpoints = ApiResource(
+      self,
+      'checkpoints',
+      endpoints=[
+        ApiEndpoint(None, Checkpoint, 'POST', 'create'),
+        ApiEndpoint(None, object_or_paginated_objects(Checkpoint), 'GET', 'fetch')
+      ]
+    )
+
+    training_runs = ApiResource(
+      self,
+      'training_runs',
+      endpoints=[
+        ApiEndpoint(None, TrainingRun, 'POST', 'create'),
+        ApiEndpoint(None, object_or_paginated_objects(TrainingRun), 'GET', 'fetch')
+      ],
+      resources=[checkpoints]
+    )
+
     self.experiments = ApiResource(
       self,
       'experiments',
@@ -87,12 +142,16 @@ class ConnectionImpl(object):
         ApiEndpoint(None, None, 'DELETE', 'delete'),
       ],
       resources=[
-        suggestions,
-        observations,
-        tokens,
         best_assignments,
+        importances,
+        metric_importances,
+        observations,
+        queued_suggestions,
         stopping_criteria,
-      ]
+        suggestions,
+        tokens,
+        training_runs,
+      ],
     )
 
     client_experiments = ApiResource(
@@ -100,7 +159,28 @@ class ConnectionImpl(object):
       'experiments',
       endpoints=[
         ApiEndpoint(None, Experiment, 'POST', 'create'),
-        ApiEndpoint(None, lambda *args, **kwargs: Pagination(Experiment, *args, **kwargs), 'GET', 'fetch'),
+        ApiEndpoint(None, paginated_objects(Experiment), 'GET', 'fetch'),
+      ],
+    )
+
+    client_project_experiments = ApiResource(
+      self,
+      'experiments',
+      endpoints=[
+        ApiEndpoint(None, paginated_objects(Experiment), 'GET', 'fetch'),
+      ],
+    )
+
+    client_projects = ApiResource(
+      self,
+      'projects',
+      endpoints=[
+        ApiEndpoint(None, Project, 'POST', 'create'),
+        ApiEndpoint(None, object_or_paginated_objects(Project), 'GET', 'fetch'),
+        ApiEndpoint(None, Project, 'PUT', 'update'),
+      ],
+      resources=[
+        client_project_experiments,
       ],
     )
 
@@ -112,44 +192,58 @@ class ConnectionImpl(object):
       ],
       resources=[
         client_experiments,
+        client_projects,
         plan,
+        tokens,
       ],
     )
 
-  def _get(self, url, params=None):
-    request_params = self._request_params(params)
-    return self.requestor.get(
-      url,
-      params=request_params,
+    self.organizations = ApiResource(
+      self,
+      'organizations',
+      endpoints=[
+        ApiEndpoint(None, object_or_paginated_objects(Organization), 'GET', 'fetch'),
+      ],
     )
 
+    self.pki_sessions = ApiResource(
+      self,
+      'pki_sessions',
+      endpoints=[
+        ApiEndpoint(None, Session, 'POST', 'create'),
+      ],
+    )
+
+  def _request(self, method, url, params):
+    if method.upper() in ('GET', 'DELETE'):
+      json, params = None, self._request_params(params)
+    else:
+      json, params = ApiObject.as_json(params), None
+    return self.requestor.request(
+      method,
+      url,
+      json=json,
+      params=params,
+    )
+
+  def _get(self, url, params=None):
+    return self._request('GET', url, params)
+
   def _post(self, url, params=None):
-    request_params = ApiObject.as_json(params)
-    return self.requestor.post(
-      url,
-      json=request_params,
-    )
+    return self._request('POST', url, params)
 
   def _put(self, url, params=None):
-    request_params = ApiObject.as_json(params)
-    return self.requestor.put(
-      url,
-      json=request_params,
-    )
+    return self._request('PUT', url, params)
 
   def _delete(self, url, params=None):
-    request_params = ApiObject.as_json(params)
-    return self.requestor.delete(
-      url,
-      params=request_params,
-    )
+    return self._request('DELETE', url, params)
 
   def _request_params(self, params):
     req_params = params or {}
 
     def serialize(value):
-      if isinstance(value, dict) or isinstance(value, list):
-        return json.dumps(value)
+      if isinstance(value, (dict, list)):
+        return simplejson.dumps(value)
       return str(value)
 
     return dict((
@@ -168,6 +262,15 @@ class ConnectionImpl(object):
   def set_proxies(self, proxies):
     self.requestor.proxies = proxies
 
+  def set_timeout(self, timeout):
+    self.requestor.timeout = timeout
+
+  def set_client_ssl_certs(self, client_ssl_certs):
+    self.requestor.client_ssl_certs = client_ssl_certs
+
+  def set_client_token(self, client_token):
+    self.requestor.set_client_token(client_token)
+
 
 class Connection(object):
   """
@@ -176,6 +279,7 @@ class Connection(object):
   """
   def __init__(self, client_token=None, user_agent=None):
     client_token = client_token or os.environ.get('SIGOPT_API_TOKEN')
+    api_url = os.environ.get('SIGOPT_API_URL') or DEFAULT_API_URL
     if not client_token:
       raise ValueError('Must provide client_token or set environment variable SIGOPT_API_TOKEN')
 
@@ -189,7 +293,7 @@ class Connection(object):
       '',
       default_headers,
     )
-    self.impl = ConnectionImpl(requestor)
+    self.impl = ConnectionImpl(requestor, api_url=api_url)
 
   def set_api_url(self, api_url):
     self.impl.set_api_url(api_url)
@@ -200,6 +304,15 @@ class Connection(object):
   def set_proxies(self, proxies):
     self.impl.set_proxies(proxies)
 
+  def set_timeout(self, timeout):
+    self.impl.set_timeout(timeout)
+
+  def set_client_ssl_certs(self, client_ssl_certs):
+    self.impl.set_client_ssl_certs(client_ssl_certs)
+
+  def set_client_token(self, client_token):
+    self.impl.set_client_token(client_token)
+
   @property
   def clients(self):
     return self.impl.clients
@@ -208,6 +321,19 @@ class Connection(object):
   def experiments(self):
     return self.impl.experiments
 
+  @property
+  def organizations(self):
+    return self.impl.organizations
+
+  @property
+  def pki_sessions(self):
+    return self.impl.pki_sessions
+
+def paginated_objects(api_object):
+  def decorator(body, *args, **kwargs):
+    return Pagination(api_object, body, *args, **kwargs)
+  return decorator
+
 
 # Allows response to be a single object of class some_class or a paginated
 # response of objects that come from class some_class
@@ -215,6 +341,5 @@ def object_or_paginated_objects(api_object):
   def decorator(body, *args, **kwargs):
     if body.get('object') == 'pagination':
       return Pagination(api_object, body, *args, **kwargs)
-    else:
-      return api_object(body, *args, **kwargs)
+    return api_object(body, *args, **kwargs)
   return decorator
