@@ -3,6 +3,7 @@ import pytest
 import mock
 
 from sigopt.interface import Connection
+from sigopt.requestor import DEFAULT_HTTP_TIMEOUT
 from sigopt.resource import ApiResource
 
 class TestInterface(object):
@@ -11,6 +12,7 @@ class TestInterface(object):
     assert conn.impl.api_url == 'https://api.sigopt.com'
     assert conn.impl.requestor.verify_ssl_certs is True
     assert conn.impl.requestor.proxies is None
+    assert conn.impl.requestor.timeout == DEFAULT_HTTP_TIMEOUT
     assert isinstance(conn.clients, ApiResource)
     assert isinstance(conn.experiments, ApiResource)
 
@@ -23,6 +25,11 @@ class TestInterface(object):
     conn.set_api_url('https://api-test.sigopt.com')
     assert conn.impl.api_url == 'https://api-test.sigopt.com'
 
+  def test_api_url_env(self):
+    with mock.patch.dict(os.environ, {'SIGOPT_API_URL': 'https://api-env.sigopt.com'}):
+      conn = Connection('client_token')
+      assert conn.impl.api_url == 'https://api-env.sigopt.com'
+
   def test_verify(self):
     conn = Connection('client_token')
     conn.set_verify_ssl_certs(False)
@@ -34,5 +41,11 @@ class TestInterface(object):
     assert conn.impl.requestor.proxies['http'] == 'http://127.0.0.1:6543'
 
   def test_error(self):
-    with pytest.raises(ValueError):
-      Connection()
+    with mock.patch.dict(os.environ, {'SIGOPT_API_TOKEN': ''}):
+      with pytest.raises(ValueError):
+        Connection()
+
+  def test_timeout(self):
+    conn = Connection('client_token')
+    conn.set_timeout(30)
+    assert conn.impl.requestor.timeout == 30
